package ru.dbotthepony.mc.otm.menu.widget

import ru.dbotthepony.mc.otm.network.DelegateSyncher
import ru.dbotthepony.kommons.util.getValue
import ru.dbotthepony.mc.otm.capability.AbstractProfiledStorage
import ru.dbotthepony.mc.otm.capability.energy.IMatteryEnergyStorage
import ru.dbotthepony.mc.otm.capability.matter.IMatterStorage
import ru.dbotthepony.mc.otm.core.collect.SupplierList
import ru.dbotthepony.mc.otm.core.immutableList
import ru.dbotthepony.mc.otm.core.math.Decimal
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.MatteryPoweredMenu
import ru.dbotthepony.mc.otm.network.StreamCodecs
import java.util.function.IntSupplier

class ProfiledLevelGaugeWidget<P : AbstractProfiledStorage<*>>(synchronizer: DelegateSyncher, val gauge: LevelGaugeWidget = LevelGaugeWidget(synchronizer)) {
	var parent: P? = null
		private set

	val historyReceive = immutableList(AbstractProfiledStorage.HISTORY_SIZE) {
		synchronizer.computed({ parent?.historyReceive?.get(it) ?: Decimal.ZERO }, StreamCodecs.DECIMAL)
	}

	val historyTransfer = immutableList(AbstractProfiledStorage.HISTORY_SIZE) {
		synchronizer.computed({ parent?.historyTransfer?.get(it) ?: Decimal.ZERO }, StreamCodecs.DECIMAL)
	}

	val directHistoryReceive = SupplierList(historyReceive)
	val directHistoryTransfer = SupplierList(historyTransfer)

	val tick by synchronizer.computedInt(IntSupplier { parent?.tick ?: 0 })
	val lastTickReceive by synchronizer.computed({ parent?.lastTickReceive ?: Decimal.ZERO }, StreamCodecs.DECIMAL)
	val lastTickTransfer by synchronizer.computed({ parent?.lastTickTransfer ?: Decimal.ZERO }, StreamCodecs.DECIMAL)

	constructor(synchronizer: DelegateSyncher, storage: P?, gauge: LevelGaugeWidget = LevelGaugeWidget(synchronizer)) : this(synchronizer, gauge = gauge) {
		if (storage != null) {
			with(storage)
		}
	}

	constructor(menu: MatteryMenu, storage: P?, gauge: LevelGaugeWidget = LevelGaugeWidget(menu)) : this(menu.mSynchronizer, storage, gauge = gauge)
	constructor(menu: MatteryMenu, gauge: LevelGaugeWidget = LevelGaugeWidget(menu)) : this(menu.mSynchronizer, gauge = gauge)

	constructor(menu: MatteryPoweredMenu, storage: P?) : this(menu.mSynchronizer, storage, menu.energyWidget)
	constructor(menu: MatteryPoweredMenu) : this(menu.mSynchronizer, menu.energyWidget)

	fun with(storage: P): ProfiledLevelGaugeWidget<P> {
		if (storage is IMatterStorage)
			gauge.with(storage)
		else if (storage is IMatteryEnergyStorage)
			gauge.with(storage)

		parent = storage
		return this
	}

	val weightedTransfer: Decimal get() {
		return AbstractProfiledStorage.calcWeightedAverage(directHistoryTransfer, tick)
	}

	val weightedReceive: Decimal get() {
		return AbstractProfiledStorage.calcWeightedAverage(directHistoryReceive, tick)
	}
}
